<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Model;

class Invoice extends Model
{
	protected $table = 'ab_invoice';
	protected $dates = ['date_orig','due_date'];
	protected $with = ['account.country.currency','items','paymentitems'];

	protected $appends = [
		'date_due',
		'due',
		'invoice_id_url',
		'total',
	];

	protected $visible = [
		'date_due',
		'due',
		'id',
		'invoice_id_url',
		'total',
	];

	private $_total = 0;
	private $_total_tax = 0;

	public function account()
	{
		return $this->belongsTo(Account::class);
	}

	public function items()
	{
		return $this->hasMany(InvoiceItem::class);
	}

	public function paymentitems()
	{
		return $this->hasMany(PaymentItem::class);
	}

	/** SCOPES **/

	/**
	 * Search for a record
	 *
	 * @param        $query
	 * @param string $term
	 * @return
	 */
	public function scopeSearch($query,string $term)
	{
		return $query->where('id','like','%'.$term.'%');
	}

	/** ATTRIBUTES **/

	public function getDueAttribute()
	{
		return sprintf('%3.'.$this->currency()->rounding.'f',$this->total - $this->paid);
	}

	public function getDateDueAttribute()
	{
		return $this->due_date->format('Y-m-d');
	}

	public function getInvoiceDateAttribute()
	{
		return $this->date_orig->format('Y-m-d');
	}

	public function getInvoiceIdAttribute()
	{
		return sprintf('%06s',$this->id);
	}

	public function getInvoiceAccountIdAttribute()
	{
		return sprintf('%02s-%04s-%06s',$this->site_id,$this->account_id,$this->invoice_id);
	}

	public function getInvoiceIdUrlAttribute()
	{
		return sprintf('<a href="/u/invoice/%s">%s</a>',$this->id,$this->invoice_account_id);
	}

	public function getInvoiceTextAttribute()
	{
		return sprintf('Thank you for using %s for your Internet Services.',config('SITE_SETUP')->site_name);
	}

	public function getPaidAttribute()
	{
		return $this->currency()->round($this->paymentitems->sum('alloc_amt'));
	}

	public function getSubTotalAttribute()
	{
		return sprintf('%3.'.$this->currency()->rounding.'f',$this->total-$this->tax_total);
	}

	public function getTaxTotalAttribute()
	{
		if (! $this->_total_tax)
		{
			foreach ($this->items as $o)
			{
				if ($o->active)
					$this->_total_tax += $this->currency()->round($o->tax);
			}
		}

		return sprintf('%3.'.$this->currency()->rounding.'f',$this->_total_tax);
	}

	public function getTotalAttribute()
	{
		if (! $this->_total)
		{
			foreach ($this->items as $o)
			{
				if ($o->active)
					$this->_total += $this->currency()->round($o->total);
			}
		}

		return sprintf('%3.'.$this->currency()->rounding.'f',$this->_total);
	}

	public function currency()
	{
		return $this->account->country->currency;
	}

	public function products()
	{
		$return = collect();

		foreach ($this->items->groupBy('product_id') as $o)
		{
			$po = $o->first()->product;
			$po->count = count($o->pluck('service_id')->unique());

			$return->push($po);
		}

		$lo = $this->account->user->language;
		return $return->sortBy(function ($item) use ($lo) {
			return $item->name($lo);
		});
	}

	public function product_services(Product $po)
	{
		$return = collect();

		$this->items->load(['service']);

		foreach ($this->items->filter(function ($item) use ($po) {
			return $item->product_id == $po->id;
		}) as $o)
		{
			$so = $o->service;
			$return->push($so);
		};

		return $return->unique()->sortBy('name');
	}

	public function product_service_items(Product $po,Service $so)
	{
		return $this->items->filter(function ($item) use ($po,$so) {
			return $item->product_id == $po->id AND $item->service_id == $so->id;
		})->filter()->sortBy('item_type');
	}
}